use std::{
    fmt::Display,
    io::{Write, stderr},
    sync::OnceLock,
};

use anstream::AutoStream;
use anstyle::{AnsiColor, Effects, Style};
use clap::ValueEnum;

pub const ERROR: Style = AnsiColor::Red.on_default().effects(Effects::BOLD);

#[derive(Copy, Clone, Debug, PartialEq, Eq, ValueEnum)]
pub enum Color {
    Auto,
    Always,
    Never,
}

impl From<Color> for anstream::ColorChoice {
    fn from(value: Color) -> Self {
        match value {
            Color::Always => Self::Always,
            Color::Never => Self::Never,
            Color::Auto => Self::Auto,
        }
    }
}

#[derive(Debug)]
pub struct Console(anstream::ColorChoice);

impl Console {
    fn new(color: Color) -> Self {
        Self(color.into())
    }

    pub fn print(&self, status: &str, style: &Style, msg: impl Display) {
        let mut stream = AutoStream::new(stderr(), self.0).lock();
        let _ = writeln!(stream, "{style}{status}{style:#} {msg}");
    }
}

static CONSOLE: OnceLock<Console> = OnceLock::new();

pub fn console() -> &'static Console {
    CONSOLE.get().expect("console should be initialized")
}

pub fn init(color: Color) {
    CONSOLE
        .set(Console::new(color))
        .expect("console should only be initialized once")
}

macro_rules! error {
    ($arg:expr) => (
        crate::console::console().print("error:", &crate::console::ERROR, ($arg))
    );

    ($($arg:tt)+) => (
        crate::console::console().print(
            "error:",
            &crate::console::ERROR,
            format_args!($($arg)+),
        )
    );
}

pub(crate) use error;
