use std::{env::consts::DLL_EXTENSION, fs::create_dir_all, path::Path, process::ExitCode};

use clap::Parser;
use clap_cargo::style::CLAP_STYLING;
use git2::Repository;

mod cc;
mod console;
mod languages;
mod settings;

use console::{Color, error};
use languages::{InstallInfo, get_install_info};
use settings::{Settings, get_settings};

#[derive(Debug, Parser)]
#[command(version, about, long_about = None)]
#[command(styles = CLAP_STYLING)]
struct Cli {
    /// when to use terminal colors
    #[arg(short, long, value_enum, default_value_t = Color::Auto)]
    color: Color,

    language: String,
}

fn main() -> ExitCode {
    let cli = Cli::parse();

    console::init(cli.color);

    if let Err(e) = main_inner(cli) {
        error!(e);
        ExitCode::FAILURE
    } else {
        ExitCode::SUCCESS
    }
}

fn main_inner(Cli { language, .. }: Cli) -> anyhow::Result<()> {
    let Settings {
        parser_dir,
        compiler,
    } = get_settings()?;

    let src_dir = tempfile::tempdir()?;

    create_dir_all(&parser_dir)?;

    // have to use a canonical path bc compiler runs elsewhere
    let parser_dir = parser_dir.canonicalize()?;

    Ok(install(
        &language,
        &parser_dir,
        src_dir.as_ref(),
        compiler.as_deref(),
    )?)
}

#[derive(Debug, thiserror::Error)]
enum InstallError {
    #[error("unknown language `{0}`")]
    UnknownLanguage(String),

    #[error("couldn't clone `{0}`")]
    Clone(&'static str),

    #[error(transparent)]
    Compile(#[from] cc::Error),
}

fn install(
    language: &str,
    parser_dir: &Path,
    src_dir: &Path,
    compiler: Option<&str>,
) -> Result<(), InstallError> {
    let InstallInfo { url, files } =
        get_install_info(language).ok_or(InstallError::UnknownLanguage(language.to_owned()))?;

    let repo_path = src_dir.join(language);
    Repository::clone(url, &repo_path).map_err(|_| InstallError::Clone(url))?;

    let output_path = parser_dir.join(language).with_extension(DLL_EXTENSION);

    let mut build = cc::Build::new();
    build.compiler(compiler).dir(&repo_path).input_files(files);
    build.compile(&output_path)?;

    Ok(())
}
